package com.itheima.util;

import java.util.Iterator;
import java.util.LinkedList;

/**
 * 此类实现了对队列数据结构的模仿；此类还实现了Iterable接口，为遍历队列元素提供了方便
 * 
 * @author mrng
 * 
 * @param <E>
 */
public class LoopList<E> implements Iterable<E> {
	/**
	 * 队列的默认长度
	 */
	private static final int DEFAULT_MAX_SIZE = 5;

	/**
	 * 队列的最大长度
	 */
	private int maxSize;

	/**
	 * 存储队列数据的内部容器
	 */
	private LinkedList<E> list = new LinkedList<E>();

	/**
	 * 创建一个长度为10的队列
	 */
	public LoopList() {
		this.maxSize = DEFAULT_MAX_SIZE;
	}

	/**
	 * 通过maxSiz指定此队列的最大长度
	 * 
	 * @param maxSize
	 *            队列的最大长度
	 */
	public LoopList(int maxSize) {
		this.maxSize = maxSize;
	}

	/**
	 * 添加元素，如果队列的长度已达到最大长度，则将第一个挤出队列并返回，再将新的元素添加进队列
	 * 
	 * @param e
	 *            要添加的新元素
	 * @return 如果有被挤出的元素则将其返回，否则返回null
	 */
	public E add(E e) {
		// 如果队列的长度已达到最大长度，则将第一个挤出队列并返回，再将新的元素添加进队列
		if (list.size() == maxSize) {
			list.add(e);
			return list.remove(0);
		}

		list.add(e);
		return null;
	}

	/**
	 * 移除此队列中的所有元素
	 */
	public void clear() {
		list.clear();
	}

	/**
	 * 如果此队列包含指定元素，则返回 true
	 * 
	 * @param obj
	 *            要测试在此队列中是否存在的元素
	 * @return 如果此队列包含指定元素，则返回 true
	 */
	public boolean contains(Object obj) {
		return list.contains(obj);
	}

	/**
	 * 获取但不移除此队列的第一个元素；如果此队列为空，则返回null
	 * 
	 * @return 队列的第一个元素，如果此列表为空，则返回null
	 */
	public E getFirst() {
		return list.peekFirst();
	}

	/**
	 * 获取但不移除此队列的最后一个元素；如果此队列为空，则返回null
	 * 
	 * @return 队列的最后一个元素，如果此列表为空，则返回null
	 */
	public E getLast() {
		return list.peekLast();
	}

	/**
	 * 移除并返回此队列的第一个元素；如果此队列为空，则返回null
	 * 
	 * @return 队列的第一个元素，如果此列表为空，则返回null
	 */
	public E removeFirst() {
		return list.pollFirst();
	}

	/**
	 * 移除并返回此队列的最后一个元素；如果此队列为空，则返回null
	 * 
	 * @return 队列的最后一个元素，如果此列表为空，则返回null
	 */
	public E removeLast() {
		return list.pollLast();
	}

	/**
	 * 获取队列中指定位置上的元素
	 * 
	 * @param index
	 *            - 要返回的元素的索引
	 * @return 队列中指定位置的元素
	 * 
	 * @exception IndexOutOfBoundsException
	 *                如果索引超出范围 (index < 0 || index >= size())
	 */
	public E getChildAt(int index) {
		return list.get(index);
	}

	/**
	 * 将此队列中指定位置的元素替换为指定的元素
	 * 
	 * @param index
	 *            要替换的元素的索引
	 * @param element
	 *            要在指定位置存储的元素
	 * @return 返回以前在指定位置上的元素
	 * 
	 * @exception IndexOutOfBoundsException
	 *                如果索引超出范围 (index < 0 || index >= size())
	 */
	public E set(int index, E element) {
		return list.set(index, element);
	}

	/**
	 * 按队列中的顺序返回一个包含所有元素的数组
	 * 
	 * @return 按队列中的顺序返回一个包含所有元素的数组
	 */
	public Object[] toArray() {
		return list.toArray();
	}

	/**
	 * 按队列中的顺序返回一个包含所有元素的数组 返回数组的运行时类型为指定数组的类型。 如果指定数组能容纳队列，则在其中返回该队列。
	 * 否则，分配具有指定数组的运行时类型和此队列大小的新数组。
	 * 
	 * @param a
	 *            要在其中存储队列元素的数组（如果它足够大）；否则，分配具有指定数组的运行时类型和此队列大小的新数组
	 * @return 按队列中的顺序返回一个包含所有元素的数组
	 * 
	 * @exception ArrayStoreException
	 *                如果指定数组的运行时类型不是此队列中每个元素的运行时类型的超类型
	 * @exception NullPointerException
	 *                如果指定的数组为 null
	 */
	public <T> T[] toArray(T[] a) {
		return list.toArray(a);
	}

	/**
	 * 获取队列的长度
	 * 
	 * @return 队列的长度
	 */
	public int getSize() {
		return list.size();
	}

	/**
	 * 返回次队列的最大长度
	 * 
	 * @return 此队列的最大长度
	 */
	public int getMaxSize() {
		return this.maxSize;
	}

	/**
	 * 如果此队列中没有元素，则返回true
	 * 
	 * @return 如果此队列中没有元素，则返回true
	 */
	public boolean isEmpty() {
		return list.isEmpty();
	}

	/**
	 * 返回此队列的字符串表现形式
	 */
	public String toString() {
		return list.toString();
	}

	/**
	 * 返回此队列的迭代器
	 */
	public Iterator<E> iterator() {
		return list.iterator();
	}
}
